rm(list=ls())

#function for scaling with loop
my.scale <- function(X){
 #scale one variable
 one.scale <- function(x){
  n <- length(x)
  moy <- mean(x)
  et <- sqrt((n-1)/n*var(x))
  y <- numeric(length(x))
  for (i in 1:length(x)){
   y[i] <- (x[i] - moy)/et
  }
  return(y)
 }
 #scale all the variables
 Y <- as.data.frame(lapply(X,one.scale))
 return(Y)
}

#function for scaling without loop
R.scale <- function(X){
 #scale one variable
 one.scale <- function(x){
  n <- length(x)
  moy <- mean(x)
  et <- sqrt((n-1)/n*var(x))
  y <- (x-moy)/et
  return(y)
 }
 #scale all the variables
 Y <- as.data.frame(lapply(X,one.scale))
 return(Y)
}

#my function for correlation matrix
#the columns are already scaled
my.correlation <- function(X){
 Y <- as.matrix(X)
 TY <- t(Y)
 n <- nrow(X)
 p <- ncol(X)
 M <- matrix(0,p,p)
 for (i in 1:p){
  for (j in 1:p){
   for (k in 1:n){
    M[i,j] <- M[i,j] + TY[i,k]*Y[k,j]
   }
  }
 }
 M <- M/n
 return(M)
}

#R function for correlation matrix
#the columns are already scaled
R.correlation <- function(X){
 Y <- as.matrix(X)
 TY <- t(Y)
 M <- TY%*%Y
 M <- M/nrow(X)
 return(M)
}

#generate a dataset
set.seed(1)
n <- 50000
p <- 50
dataset <- as.data.frame(matrix(runif(n*p),n,p))

#checking the functions
my.Z <- my.scale(dataset)
print(sapply(my.Z,mean))

R.Z <- R.scale(dataset)
print(sapply(R.Z,mean))

my.R <- my.correlation(my.Z)
print(my.R)

R.R <- R.correlation(R.Z)
print(R.R)

#load package
library(rbenchmark)

#benchmarking scale
benchmark(my.scale(dataset),R.scale(dataset),columns=c("elapsed","relative"),order="relative", replications=10)

#benchmarking correlations
benchmark(my.correlation(my.Z),R.correlation(my.Z),columns=c("elapsed","relative"),order="relative", replications=10)

#**********************************
#Compiled versions of our functions
#**********************************

#load package
library(compiler)

#my scale compiled
comp.my.scale <- cmpfun(my.scale)

#R scale compiled
comp.R.scale <- cmpfun(R.scale)

#my correlation compiled
comp.my.correlation <- cmpfun(my.correlation)

#R correlation compiled
comp.R.correlation <- cmpfun(R.correlation)

#benchmarking scale compiled functions
benchmark(comp.my.scale(dataset),comp.R.scale(dataset),columns=c("elapsed","relative"),order="relative",replications=10)

#benchmarking correlation compiled functions
benchmark(comp.my.correlation(my.Z),comp.R.correlation(my.Z),columns=c("elapsed","relative"),order="relative", replications=10)

#benchmarking cor function
benchmark(cor(dataset),replications=10)

